@@ -1,19 +1,20 @@
 /*
- bctoolbox
- Copyright (C) 2016  Belledonne Communications SARL
- 
- This program is free software: you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation, either version 2 of the License, or
- (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * Copyright (c) 2016-2020 Belledonne Communications SARL.
+ *
+ * This file is part of bctoolbox.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -154,14 +155,14 @@ extern "C" bctbx_iterator_t *bctbx_map_cchar_end(const bctbx_map_t *map) {
 	return bctbx_map_end_type<mmap_cchar_t>(map);
 }
 
-template<typename T> bctbx_iterator_t * bctbx_map_find_key_type(bctbx_map_t *map, typename T::key_type key) {
+template<typename T> bctbx_iterator_t * bctbx_map_find_key_type(const bctbx_map_t *map, typename T::key_type key) {
 	bctbx_iterator_t * it = (bctbx_iterator_t*) new typename T::iterator(((T *)map)->find(key));
 	return it;
 }
-extern "C" bctbx_iterator_t * bctbx_map_ullong_find_key(bctbx_map_t *map, unsigned long long key) {
+extern "C" bctbx_iterator_t * bctbx_map_ullong_find_key(const bctbx_map_t *map, unsigned long long key) {
 	return bctbx_map_find_key_type<mmap_ullong_t>(map, (mmap_ullong_t::key_type)key);
 }
-extern "C" bctbx_iterator_t * bctbx_map_cchar_find_key(bctbx_map_t *map, const char * key) {
+extern "C" bctbx_iterator_t * bctbx_map_cchar_find_key(const bctbx_map_t *map, const char * key) {
 	return bctbx_map_find_key_type<mmap_cchar_t>(map, (mmap_cchar_t::key_type)key);
 }
 
@@ -175,7 +176,7 @@ extern "C" size_t bctbx_map_cchar_size(const bctbx_map_t *map) {
 	return bctbx_map_size_type<mmap_cchar_t>(map);
 }
 
-extern "C" bctbx_iterator_t * bctbx_map_ullong_find_custom(bctbx_map_t *map, bctbx_compare_func compare_func, const void *user_data) {
+extern "C" bctbx_iterator_t * bctbx_map_ullong_find_custom(const bctbx_map_t *map, bctbx_compare_func compare_func, const void *user_data) {
 	bctbx_iterator_t * end = bctbx_map_ullong_end(map);
 	
 	for(bctbx_iterator_t * it = bctbx_map_ullong_begin(map);!bctbx_iterator_ullong_equals(it,end);) {
@@ -190,7 +191,7 @@ extern "C" bctbx_iterator_t * bctbx_map_ullong_find_custom(bctbx_map_t *map, bct
 	bctbx_iterator_ullong_delete(end);
 	return NULL;
 }
-extern "C" bctbx_iterator_t * bctbx_map_cchar_find_custom(bctbx_map_t *map, bctbx_compare_func compare_func, const void *user_data) {
+extern "C" bctbx_iterator_t * bctbx_map_cchar_find_custom(const bctbx_map_t *map, bctbx_compare_func compare_func, const void *user_data) {
 	bctbx_iterator_t * end = bctbx_map_cchar_end(map);
 	
 	for(bctbx_iterator_t * it = bctbx_map_cchar_begin(map);!bctbx_iterator_cchar_equals(it,end);) {
@@ -269,7 +270,7 @@ extern "C" bctbx_pair_cchar_t * bctbx_pair_cchar_new(const char * key,void *valu
 	return (bctbx_pair_cchar_t *)bctbx_pair_new<mmap_cchar_t>((mmap_cchar_t::key_type)key, value);
 }
 
-template<typename T> typename T::key_type bctbx_pair_get_first(const typename T::value_type  * pair) {
+template<typename T> const typename T::key_type& bctbx_pair_get_first(const typename T::value_type  * pair) {
 	return ((typename T::value_type*)pair)->first;
 }
 extern "C" unsigned long long bctbx_pair_ullong_get_first(const bctbx_pair_ullong_t * pair) {
