@@ -1,21 +1,21 @@
 /*
-crypto_mbedtls.c
-Copyright (C) 2016  Belledonne Communications SARL
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2016-2020 Belledonne Communications SARL.
+ *
+ * This file is part of bctoolbox.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -27,9 +27,8 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <mbedtls/ssl.h>
 #include <mbedtls/timing.h>
 #include <mbedtls/error.h>
-#include <mbedtls/net.h>
 #include <mbedtls/base64.h>
-
+#include <mbedtls/version.h>
 #include <mbedtls/pem.h>
 #include <mbedtls/x509.h>
 #include <mbedtls/entropy.h>
@@ -40,23 +39,28 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <mbedtls/sha512.h>
 #include <mbedtls/gcm.h>
 
-#include <bctoolbox/crypto.h>
+#if MBEDTLS_VERSION_NUMBER >= 0x02040000 // v2.4.0
+#include <mbedtls/net_sockets.h>
+#else
+#include <mbedtls/net.h>
+#endif
+
+#include "bctoolbox/crypto.h"
 #include "bctoolbox/logging.h"
 
 
-static int bctbx_ssl_sendrecv_callback_return_remap(int32_t ret_code) {
-	switch (ret_code) {
-		case BCTBX_ERROR_NET_WANT_READ:
-			return MBEDTLS_ERR_SSL_WANT_READ;
-		case BCTBX_ERROR_NET_WANT_WRITE:
-			return MBEDTLS_ERR_SSL_WANT_WRITE;
-		case BCTBX_ERROR_NET_CONN_RESET:
-			return MBEDTLS_ERR_NET_CONN_RESET;
-		default:
-			return (int)ret_code;
-	}
+/*** Cleaning ***/
+/**
+ * @brief force a buffer value to zero in a way that shall prevent the compiler from optimizing it out
+ *
+ * @param[in/out]	buffer	the buffer to be cleared
+ * @param[in]		size	buffer size
+ */
+void bctbx_clean(void *buffer, size_t size) {
+	mbedtls_platform_zeroize(buffer, size);
 }
 
+/*** Error code translation ***/
 void bctbx_strerror(int32_t error_code, char *buffer, size_t buffer_length) {
 	if (error_code>0) {
 		snprintf(buffer, buffer_length, "%s", "Invalid Error code");
@@ -74,6 +78,7 @@ void bctbx_strerror(int32_t error_code, char *buffer, size_t buffer_length) {
 	return;
 }
 
+/*** base64 ***/
 int32_t bctbx_base64_encode(unsigned char *output, size_t *output_length, const unsigned char *input, size_t input_length) {
 	size_t byte_written = 0;
 	int ret = mbedtls_base64_encode(output, *output_length, &byte_written, input, input_length);
@@ -144,10 +149,9 @@ char *bctbx_signing_key_get_pem(bctbx_signing_key_t *key) {
 
 int32_t bctbx_signing_key_parse(bctbx_signing_key_t *key, const char *buffer, size_t buffer_length, const unsigned char *password, size_t password_length) {
 	int err;
+	
 	err=mbedtls_pk_parse_key((mbedtls_pk_context *)key, (const unsigned char *)buffer, buffer_length, password, password_length);
-	if(err==0 && !mbedtls_pk_can_do((mbedtls_pk_context *)key, MBEDTLS_PK_RSA)) {
-		err=MBEDTLS_ERR_PK_TYPE_MISMATCH;
-	}
+	
 	if (err<0) {
 		char tmp[128];
 		mbedtls_strerror(err,tmp,sizeof(tmp));
@@ -159,10 +163,9 @@ int32_t bctbx_signing_key_parse(bctbx_signing_key_t *key, const char *buffer, si
 
 int32_t bctbx_signing_key_parse_file(bctbx_signing_key_t *key, const char *path, const char *password) {
 	int err;
+	
 	err=mbedtls_pk_parse_keyfile((mbedtls_pk_context *)key, path, password);
-	if(err==0 && !mbedtls_pk_can_do((mbedtls_pk_context *)key,MBEDTLS_PK_RSA)) {
-		err=MBEDTLS_ERR_PK_TYPE_MISMATCH;
-	}
+	
 	if (err<0) {
 		char tmp[128];
 		mbedtls_strerror(err,tmp,sizeof(tmp));
@@ -232,7 +235,7 @@ int32_t bctbx_x509_certificate_get_der(bctbx_x509_certificate_t *cert, unsigned
 	return 0;
 }
 
-int32_t bctbx_x509_certificate_get_subject_dn(bctbx_x509_certificate_t *cert, char *dn, size_t dn_length) {
+int32_t bctbx_x509_certificate_get_subject_dn(const bctbx_x509_certificate_t *cert, char *dn, size_t dn_length) {
 	if (cert==NULL) {
 		return BCTBX_ERROR_INVALID_CERTIFICATE;
 	}
@@ -240,6 +243,34 @@ int32_t bctbx_x509_certificate_get_subject_dn(bctbx_x509_certificate_t *cert, ch
 	return mbedtls_x509_dn_gets(dn, dn_length, &(((mbedtls_x509_crt *)cert)->subject));
 }
 
+
+bctbx_list_t *bctbx_x509_certificate_get_subjects(const bctbx_x509_certificate_t *cert){
+	bctbx_list_t *ret = NULL;
+	char subject[1024]={0};
+	const mbedtls_x509_sequence *subjectAltNames = &((mbedtls_x509_crt *)cert)->subject_alt_names;
+	
+	for (; subjectAltNames != NULL; subjectAltNames = subjectAltNames->next){
+		const mbedtls_asn1_buf *buf = &subjectAltNames->buf;
+		if (buf->tag == ( MBEDTLS_ASN1_CONTEXT_SPECIFIC | 2 ) ){
+			if (buf->p){
+				ret = bctbx_list_append(ret, bctbx_strndup((char*)buf->p, buf->len));
+			}
+		}
+	}
+	
+	if (bctbx_x509_certificate_get_subject_dn(cert, subject, sizeof(subject)-1) > 0){
+		char *cn = strstr(subject, "CN=");
+		if (cn){
+			char *end;
+			cn += 3;
+			end = strchr(cn, ',');
+			if (end) *end = '\0';
+			ret = bctbx_list_append(ret, bctbx_strdup(cn));
+		}
+	}
+	return ret;
+}
+
 int32_t bctbx_x509_certificate_generate_selfsigned(const char *subject, bctbx_x509_certificate_t *certificate, bctbx_signing_key_t *pkey, char * pem, size_t pem_length) {
 	mbedtls_entropy_context entropy;
 	mbedtls_ctr_drbg_context ctr_drbg;
@@ -442,7 +473,7 @@ int32_t bctbx_x509_certificate_get_fingerprint(const bctbx_x509_certificate_t *c
 		break;
 
 		case MBEDTLS_MD_SHA512:
-			mbedtls_sha512(crt->raw.p, crt->raw.len, buffer, 1); /* last argument is a boolean, indicate to output sha-384 and not sha-512 */
+			mbedtls_sha512(crt->raw.p, crt->raw.len, buffer, 0); /* last argument is a boolean, indicate to output sha-384 and not sha-512 */
 			hash_length = 64;
 			hash_alg_string="SHA-512";
 		break;
@@ -754,7 +785,7 @@ void bctbx_DHMComputeSecret(bctbx_DHMContext_t *context, int (*rngFunction)(void
 	mbedtls_dhm_calc_secret((mbedtls_dhm_context *)(context->cryptoModuleData), sharedSecretBuffer, 384, &keyLength, (int (*)(void *, unsigned char *, size_t))rngFunction, rngContext);
 	/* now copy the resulting secret in the correct place in buffer(result may actually miss some front zero bytes, real length of output is now in keyLength but we want primeLength bytes) */
 	memcpy(context->key+(context->primeLength-keyLength), sharedSecretBuffer, keyLength);
-	memset(sharedSecretBuffer, 0, 384); /* purge secret from temporary buffer */
+	bctbx_clean(sharedSecretBuffer, 384); /* purge secret from temporary buffer */
 }
 
 /* clean DHM context */
@@ -762,12 +793,12 @@ void bctbx_DestroyDHMContext(bctbx_DHMContext_t *context) {
 	if (context!= NULL) {
 		/* key and secret must be erased from memory and not just freed */
 		if (context->secret != NULL) {
-			memset(context->secret, 0, context->secretLength);
+			bctbx_clean(context->secret, context->secretLength);
 			free(context->secret);
 		}
 		free(context->self);
 		if (context->key != NULL) {
-			memset(context->key, 0, context->primeLength);
+			bctbx_clean(context->key, context->primeLength);
 			free(context->key);
 		}
 		free(context->peer);
@@ -780,6 +811,20 @@ void bctbx_DestroyDHMContext(bctbx_DHMContext_t *context) {
 }
 
 /*** SSL Client ***/
+
+static int bctbx_ssl_sendrecv_callback_return_remap(int32_t ret_code) {
+	switch (ret_code) {
+		case BCTBX_ERROR_NET_WANT_READ:
+			return MBEDTLS_ERR_SSL_WANT_READ;
+		case BCTBX_ERROR_NET_WANT_WRITE:
+			return MBEDTLS_ERR_SSL_WANT_WRITE;
+		case BCTBX_ERROR_NET_CONN_RESET:
+			return MBEDTLS_ERR_NET_CONN_RESET;
+		default:
+			return (int)ret_code;
+	}
+}
+
 /*
  * Default profile used to configure ssl_context, allow 1024 bits keys(while mbedtls default is 2048)
  */
@@ -943,6 +988,10 @@ const char *bctbx_ssl_get_ciphersuite(bctbx_ssl_context_t *ssl_ctx){
 	return mbedtls_ssl_get_ciphersuite(&(ssl_ctx->ssl_ctx));
 }
 
+int bctbx_ssl_get_ciphersuite_id(const char *ciphersuite){
+	return mbedtls_ssl_get_ciphersuite_id(ciphersuite);
+}
+
 const char *bctbx_ssl_get_version(bctbx_ssl_context_t *ssl_ctx){
 	return mbedtls_ssl_get_version(&(ssl_ctx->ssl_ctx));
 }
@@ -957,7 +1006,12 @@ uint8_t bctbx_dtls_srtp_supported(void) {
 	return 1;
 }
 
-static bctbx_dtls_srtp_profile_t bctbx_srtp_profile_mbedtls2bctoolbox(enum mbedtls_DTLS_SRTP_protection_profiles mbedtls_profile) {
+void bctbx_ssl_set_mtu(bctbx_ssl_context_t *ssl_ctx, uint16_t mtu) {
+	// remove the record expansion to the given MTU
+	mbedtls_ssl_set_mtu(&(ssl_ctx->ssl_ctx), mtu - mbedtls_ssl_get_record_expansion(&(ssl_ctx->ssl_ctx)));
+}
+
+static bctbx_dtls_srtp_profile_t bctbx_srtp_profile_mbedtls2bctoolbox(mbedtls_ssl_srtp_profile mbedtls_profile) {
 	switch (mbedtls_profile) {
 		case MBEDTLS_SRTP_AES128_CM_HMAC_SHA1_80:
 			return BCTBX_SRTP_AES128_CM_HMAC_SHA1_80;
@@ -972,7 +1026,7 @@ static bctbx_dtls_srtp_profile_t bctbx_srtp_profile_mbedtls2bctoolbox(enum mbedt
 	}
 }
 
-static enum mbedtls_DTLS_SRTP_protection_profiles bctbx_srtp_profile_bctoolbox2mbedtls(bctbx_dtls_srtp_profile_t bctbx_profile) {
+static mbedtls_ssl_srtp_profile bctbx_srtp_profile_bctoolbox2mbedtls(bctbx_dtls_srtp_profile_t bctbx_profile) {
 	switch (bctbx_profile) {
 		case BCTBX_SRTP_AES128_CM_HMAC_SHA1_80:
 			return MBEDTLS_SRTP_AES128_CM_HMAC_SHA1_80;
@@ -1017,6 +1071,8 @@ uint8_t bctbx_dtls_srtp_supported(void) {
 	return 0;
 }
 
+void bctbx_ssl_set_mtu(bctbx_ssl_context_t *ssl_ctx, uint16_t mtu) { }
+
 bctbx_dtls_srtp_profile_t bctbx_ssl_get_dtls_srtp_protection_profile(bctbx_ssl_context_t *ssl_ctx) {
 	return BCTBX_SRTP_UNDEFINED;
 }
@@ -1051,6 +1107,10 @@ bctbx_ssl_config_t *bctbx_ssl_config_new(void) {
 	return ssl_config;
 }
 
+bctbx_type_implementation_t bctbx_ssl_get_implementation_type(void) {
+	return BCTBX_MBEDTLS;
+}
+
 int32_t bctbx_ssl_config_set_crypto_library_config(bctbx_ssl_config_t *ssl_config, void *internal_config) {
 	if (ssl_config == NULL) {
 		return BCTBX_ERROR_INVALID_SSL_CONFIG;
@@ -1173,6 +1233,24 @@ int32_t bctbx_ssl_config_set_transport (bctbx_ssl_config_t *ssl_config, int tran
 	return 0;
 }
 
+int32_t bctbx_ssl_config_set_ciphersuites(bctbx_ssl_config_t *ssl_config, const int *ciphersuites) {
+	if (ssl_config == NULL) {
+	 return BCTBX_ERROR_INVALID_SSL_CONFIG;
+	 }
+	/* remap input arguments */
+	if (ciphersuites == NULL) {
+		return BCTBX_ERROR_INVALID_INPUT_DATA;
+	}
+
+	mbedtls_ssl_conf_ciphersuites(ssl_config->ssl_config, ciphersuites);
+
+	return 0;
+}
+
+void *bctbx_ssl_config_get_private_config(bctbx_ssl_config_t *ssl_config) {
+	return (void *)ssl_config->ssl_config;
+}
+
 int32_t bctbx_ssl_config_set_authmode(bctbx_ssl_config_t *ssl_config, int authmode) {
 	int mbedtls_authmode;
 	if (ssl_config == NULL) {
@@ -1252,7 +1330,7 @@ int32_t bctbx_ssl_config_set_own_cert(bctbx_ssl_config_t *ssl_config, bctbx_x509
 #ifdef HAVE_DTLS_SRTP
 int32_t bctbx_ssl_config_set_dtls_srtp_protection_profiles(bctbx_ssl_config_t *ssl_config, const bctbx_dtls_srtp_profile_t *profiles, size_t profiles_number) {
 	size_t i;
-	enum mbedtls_DTLS_SRTP_protection_profiles dtls_srtp_mbedtls_profiles[4];
+	mbedtls_ssl_srtp_profile dtls_srtp_mbedtls_profiles[4];
 
 	if (ssl_config == NULL) {
 		return BCTBX_ERROR_INVALID_SSL_CONFIG;
@@ -1310,9 +1388,64 @@ int32_t bctbx_ssl_context_setup(bctbx_ssl_context_t *ssl_ctx, bctbx_ssl_config_t
 /*****************************************************************************/
 /***** Hashing                                                           *****/
 /*****************************************************************************/
+/*
+ * @brief HMAC-SHA512 wrapper
+ * @param[in] 	key		HMAC secret key
+ * @param[in] 	keyLength	HMAC key length
+ * @param[in]	input 		Input data buffer
+ * @param[in]   inputLength	Input data length
+ * @param[in]	hmacLength	Length of output required in bytes, HMAC output is truncated to the hmacLength left bytes. 64 bytes maximum
+ * @param[out]	output		Output data buffer.
+ *
+ */
+void bctbx_hmacSha512(const uint8_t *key,
+		size_t keyLength,
+		const uint8_t *input,
+		size_t inputLength,
+		uint8_t hmacLength,
+		uint8_t *output)
+{
+	uint8_t hmacOutput[64];
+	mbedtls_md_hmac(mbedtls_md_info_from_type(MBEDTLS_MD_SHA512), key, keyLength, input, inputLength, hmacOutput);
+
+	/* check output length, can't be>64 */
+	if (hmacLength>64) {
+		memcpy(output, hmacOutput, 64);
+	} else {
+		memcpy(output, hmacOutput, hmacLength);
+	}
+}
 
 /*
- * HMAC-SHA-256 wrapper
+ * @brief HMAC-SHA384 wrapper
+ * @param[in] 	key		HMAC secret key
+ * @param[in] 	keyLength	HMAC key length
+ * @param[in]	input 		Input data buffer
+ * @param[in]   inputLength	Input data length
+ * @param[in]	hmacLength	Length of output required in bytes, HMAC output is truncated to the hmacLength left bytes. 48 bytes maximum
+ * @param[out]	output		Output data buffer.
+ *
+ */
+void bctbx_hmacSha384(const uint8_t *key,
+		size_t keyLength,
+		const uint8_t *input,
+		size_t inputLength,
+		uint8_t hmacLength,
+		uint8_t *output)
+{
+	uint8_t hmacOutput[48];
+	mbedtls_md_hmac(mbedtls_md_info_from_type(MBEDTLS_MD_SHA384), key, keyLength, input, inputLength, hmacOutput);
+
+	/* check output length, can't be>48 */
+	if (hmacLength>48) {
+		memcpy(output, hmacOutput, 48);
+	} else {
+		memcpy(output, hmacOutput, hmacLength);
+	}
+}
+
+/*
+ * brief HMAC-SHA256 wrapper
  * @param[in] 	key			HMAC secret key
  * @param[in] 	keyLength	HMAC key length
  * @param[in]	input 		Input data buffer
@@ -1339,11 +1472,59 @@ void bctbx_hmacSha256(const uint8_t *key,
 	}
 }
 
+/*
+ * @brief SHA512 wrapper
+ * @param[in]	input 		Input data buffer
+ * @param[in]   inputLength	Input data length in bytes
+ * @param[in]	hashLength	Length of output required in bytes, Output is truncated to the hashLength left bytes. 64 bytes maximum
+ * @param[out]	output		Output data buffer.
+ *
+ */
+void bctbx_sha512(const uint8_t *input,
+		size_t inputLength,
+		uint8_t hashLength,
+		uint8_t *output)
+{
+	uint8_t hashOutput[64];
+	mbedtls_sha512(input, inputLength, hashOutput, 0); /* last param to zero to select SHA512 and not SHA384 */
+
+	/* check output length, can't be>64 */
+	if (hashLength>64) {
+		memcpy(output, hashOutput, 64);
+	} else {
+		memcpy(output, hashOutput, hashLength);
+	}
+}
+
+/*
+ * @brief SHA384 wrapper
+ * @param[in]	input 		Input data buffer
+ * @param[in]   inputLength	Input data length in bytes
+ * @param[in]	hashLength	Length of output required in bytes, Output is truncated to the hashLength left bytes. 48 bytes maximum
+ * @param[out]	output		Output data buffer.
+ *
+ */
+void bctbx_sha384(const uint8_t *input,
+		size_t inputLength,
+		uint8_t hashLength,
+		uint8_t *output)
+{
+	uint8_t hashOutput[48];
+	mbedtls_sha512(input, inputLength, hashOutput, 1); /* last param to one to select SHA384 and not SHA512 */
+
+	/* check output length, can't be>48 */
+	if (hashLength>48) {
+		memcpy(output, hashOutput, 48);
+	} else {
+		memcpy(output, hashOutput, hashLength);
+	}
+}
+
 /*
  * @brief SHA256 wrapper
  * @param[in]	input 		Input data buffer
  * @param[in]   inputLength	Input data length in bytes
- * @param[in]	hmacLength	Length of output required in bytes, HMAC output is truncated to the hmacLength left bytes. 32 bytes maximum
+ * @param[in]	hashLength	Length of output required in bytes, Output is truncated to the hashLength left bytes. 32 bytes maximum
  * @param[out]	output		Output data buffer.
  *
  */
