@@ -1,21 +1,21 @@
 /*
-crypto_polarssl.c
-Copyright (C) 2016  Belledonne Communications SARL
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2016-2020 Belledonne Communications SARL.
+ *
+ * This file is part of bctoolbox.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 #include <stdlib.h>
 #include <string.h>
 #ifdef HAVE_CONFIG_H
@@ -44,6 +44,18 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #define snprintf _snprintf
 #endif
 
+/*** Cleaning ***/
+/**
+ * @brief force a buffer value to zero in a way that shall prevent the compiler from optimizing it out
+ *
+ * @param[in/out]	buffer	the buffer to be cleared
+ * @param[in]		size	buffer size
+ */
+void bctbx_clean(void *buffer, size_t size) {
+	volatile uint8_t *p = buffer;
+	while(size--) *p++ = 0;
+}
+
 static int bctbx_ssl_sendrecv_callback_return_remap(int32_t ret_code) {
 	switch (ret_code) {
 		case BCTBX_ERROR_NET_WANT_READ:
@@ -57,6 +69,10 @@ static int bctbx_ssl_sendrecv_callback_return_remap(int32_t ret_code) {
 	}
 }
 
+bctbx_type_implementation_t bctbx_ssl_get_implementation_type(void) {
+	return BCTBX_POLARSSL;
+}
+
 void bctbx_strerror(int32_t error_code, char *buffer, size_t buffer_length) {
 	if (error_code>0) {
 		snprintf(buffer, buffer_length, "%s", "Invalid Error code");
@@ -230,7 +246,7 @@ int32_t bctbx_x509_certificate_get_der(bctbx_x509_certificate_t *cert, unsigned
 	return 0;
 }
 
-int32_t bctbx_x509_certificate_get_subject_dn(bctbx_x509_certificate_t *cert, char *dn, size_t dn_length) {
+int32_t bctbx_x509_certificate_get_subject_dn(const bctbx_x509_certificate_t *cert, char *dn, size_t dn_length) {
 	if (cert==NULL) {
 		return BCTBX_ERROR_INVALID_CERTIFICATE;
 	}
@@ -238,6 +254,11 @@ int32_t bctbx_x509_certificate_get_subject_dn(bctbx_x509_certificate_t *cert, ch
 	return x509_dn_gets(dn, dn_length, &(((x509_crt *)cert)->subject));
 }
 
+bctbx_list_t *bctbx_x509_certificate_get_subjects(const bctbx_x509_certificate_t *cert){
+	bctbx_error("bctbx_x509_certificate_get_subjects(): not implemented for polarssl.");
+	return NULL;
+}
+
 int32_t bctbx_x509_certificate_generate_selfsigned(const char *subject, bctbx_x509_certificate_t *certificate, bctbx_signing_key_t *pkey, char * pem, size_t pem_length) {
 	entropy_context entropy;
 	ctr_drbg_context ctr_drbg;
@@ -441,7 +462,7 @@ int32_t bctbx_x509_certificate_get_fingerprint(const bctbx_x509_certificate_t *c
 		break;
 
 		case POLARSSL_MD_SHA512:
-			sha512(crt->raw.p, crt->raw.len, buffer, 1); /* last argument is a boolean, indicate to output sha-384 and not sha-512 */
+			sha512(crt->raw.p, crt->raw.len, buffer, 0); /* last argument is a boolean, indicate to output sha-384 and not sha-512 */
 			hash_length = 64;
 			hash_alg_string="SHA-512";
 		break;
@@ -808,6 +829,9 @@ const char *bctbx_ssl_get_ciphersuite(bctbx_ssl_context_t *ssl_ctx){
 	return ssl_get_ciphersuite(&(ssl_ctx->ssl_ctx));
 }
 
+int bctbx_ssl_get_ciphersuite_id(const char *ciphersuite){
+	return BCTBX_ERROR_UNAVAILABLE_FUNCTION;
+}
 const char *bctbx_ssl_get_version(bctbx_ssl_context_t *ssl_ctx){
 	return ssl_get_version(&(ssl_ctx->ssl_ctx));
 }
@@ -821,6 +845,9 @@ int32_t bctbx_ssl_set_hostname(bctbx_ssl_context_t *ssl_ctx, const char *hostnam
 }
 
 /** DTLS SRTP functions **/
+/* used for DTLS SRTP but not implemented on polarssl, available on mbedtls only */
+void bctbx_ssl_set_mtu(bctbx_ssl_context_t *ssl_ctx, uint16_t mtu) { };
+
 #ifdef HAVE_DTLS_SRTP
 uint8_t bctbx_dtls_srtp_supported(void) {
 	return 1;
@@ -1013,6 +1040,15 @@ int32_t bctbx_ssl_config_set_transport (bctbx_ssl_config_t *ssl_config, int tran
 	return 0;
 }
 
+int32_t bctbx_ssl_config_set_ciphersuites(bctbx_ssl_config_t *ssl_config, const int *ciphersuites) {
+	return BCTBX_ERROR_INVALID_SSL_CONFIG;
+}
+
+/* unavailable function */
+void *bctbx_ssl_config_get_private_config(bctbx_ssl_config_t *ssl_config) {
+	return NULL;
+}
+
 int32_t bctbx_ssl_config_set_authmode(bctbx_ssl_config_t *ssl_config, int authmode) {
 	if (ssl_config != NULL) {
 		switch (authmode) {
@@ -1164,6 +1200,62 @@ int32_t bctbx_ssl_context_setup(bctbx_ssl_context_t *ssl_ctx, bctbx_ssl_config_t
 /*****************************************************************************/
 /***** Hashing                                                           *****/
 /*****************************************************************************/
+/**
+ * @brief HMAC-SHA512 wrapper
+ * @param[in] 	key		HMAC secret key
+ * @param[in] 	keyLength	HMAC key length in bytes
+ * @param[in]	input 		Input data buffer
+ * @param[in]   inputLength	Input data length in bytes
+ * @param[in]	hmacLength	Length of output required in bytes, HMAC output is truncated to the hmacLength left bytes. 48 bytes maximum
+ * @param[out]	output		Output data buffer.
+ *
+ */
+void bctbx_hmacSha512(const uint8_t *key,
+		size_t keyLength,
+		const uint8_t *input,
+		size_t inputLength,
+		uint8_t hmacLength,
+		uint8_t *output)
+{
+	uint8_t hmacOutput[64];
+	sha512_hmac(key, keyLength, input, inputLength, hmacOutput, 0); /* last param to one to select SHA512 and not SHA384 */
+
+	/* check output length, can't be>64 */
+	if (hmacLength>64) {
+		memcpy(output, hmacOutput, 64);
+	} else {
+		memcpy(output, hmacOutput, hmacLength);
+	}
+}
+
+/**
+ * @brief HMAC-SHA384 wrapper
+ * @param[in] 	key		HMAC secret key
+ * @param[in] 	keyLength	HMAC key length in bytes
+ * @param[in]	input 		Input data buffer
+ * @param[in]   inputLength	Input data length in bytes
+ * @param[in]	hmacLength	Length of output required in bytes, HMAC output is truncated to the hmacLength left bytes. 48 bytes maximum
+ * @param[out]	output		Output data buffer.
+ *
+ */
+void bctbx_hmacSha384(const uint8_t *key,
+		size_t keyLength,
+		const uint8_t *input,
+		size_t inputLength,
+		uint8_t hmacLength,
+		uint8_t *output)
+{
+	uint8_t hmacOutput[48];
+	sha512_hmac(key, keyLength, input, inputLength, hmacOutput, 1); /* last param to one to select SHA384 and not SHA512 */
+
+	/* check output length, can't be>48 */
+	if (hmacLength>48) {
+		memcpy(output, hmacOutput, 48);
+	} else {
+		memcpy(output, hmacOutput, hmacLength);
+	}
+}
+
 /**
  * @brief HMAC-SHA256 wrapper
  * @param[in] 	key			HMAC secret key
@@ -1192,6 +1284,54 @@ void bctbx_hmacSha256(const uint8_t *key,
 	}
 }
 
+/*
+ * @brief SHA512 wrapper
+ * @param[in]	input 		Input data buffer
+ * @param[in]   inputLength	Input data length in bytes
+ * @param[in]	hashLength	Length of output required in bytes, Output is truncated to the hashLength left bytes. 64 bytes maximum
+ * @param[out]	output		Output data buffer.
+ *
+ */
+void bctbx_sha512(const uint8_t *input,
+		size_t inputLength,
+		uint8_t hashLength,
+		uint8_t *output)
+{
+	uint8_t hashOutput[64];
+	sha512(input, inputLength, hashOutput, 0); /* last param to zero to select SHA512 and not SHA384 */
+
+	/* check output length, can't be>64 */
+	if (hashLength>64) {
+		memcpy(output, hashOutput, 64);
+	} else {
+		memcpy(output, hashOutput, hashLength);
+	}
+}
+
+/*
+ * @brief SHA384 wrapper
+ * @param[in]	input 		Input data buffer
+ * @param[in]   inputLength	Input data length in bytes
+ * @param[in]	hashLength	Length of output required in bytes, Output is truncated to the hashLength left bytes. 48 bytes maximum
+ * @param[out]	output		Output data buffer.
+ *
+ */
+void bctbx_sha384(const uint8_t *input,
+		size_t inputLength,
+		uint8_t hashLength,
+		uint8_t *output)
+{
+	uint8_t hashOutput[48];
+	sha512(input, inputLength, hashOutput, 1); /* last param to one to select SHA384 and not SHA512 */
+
+	/* check output length, can't be>48 */
+	if (hashLength>48) {
+		memcpy(output, hashOutput, 48);
+	} else {
+		memcpy(output, hashOutput, hashLength);
+	}
+}
+
 /**
  * @brief SHA256 wrapper
  * @param[in]	input 		Input data buffer
