@@ -1,22 +1,21 @@
 /*
-bctoolbox
-Copyright (C) 2016  Belledonne Communications SARL
-
-
-This library is free software; you can redistribute it and/or
-modify it under the terms of the GNU Lesser General Public
-License as published by the Free Software Foundation; either
-version 2.1 of the License, or (at your option) any later version.
-
-This library is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-Lesser General Public License for more details.
-
-You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*/
+ * Copyright (c) 2016-2020 Belledonne Communications SARL.
+ *
+ * This file is part of bctoolbox.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -37,31 +36,77 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #endif
 #endif
 
+/*
+ * Exclude windows and android for bctbx_set_thread_log_level() implementation.
+ * Android has lots of bugs around thread local storage and JVM.
+ */
+#if !defined(_WIN32) && !defined(__ANDROID__)
+#define THREAD_LOG_LEVEL_ENABLED 1
+#endif
+
+#ifdef __ANDROID__
+#include <android/log.h>
+#endif /* __ANDROID__ */
 
 typedef struct{
 	char *domain;
 	unsigned int logmask;
+#ifdef THREAD_LOG_LEVEL_ENABLED
+	int thread_level_set; /* Set to 1 if a thread specific log level has been set. This enables an optimisation
+				only for the case of an app that never uses per-thread log levels.*/
+	pthread_key_t thread_level_key; /* The key to access the thread specific level. */
+#endif
 }BctoolboxLogDomain;
 
-static void bctbx_log_domain_destroy(BctoolboxLogDomain *obj){
+#ifdef THREAD_LOG_LEVEL_ENABLED
+static void thread_level_key_destroy(void *ptr){
+	bctbx_free(ptr);
+}
+#endif
+
+static BctoolboxLogDomain * bctbx_log_domain_new(const char *domain, unsigned int logmask){
+	BctoolboxLogDomain *ld = bctbx_new0(BctoolboxLogDomain, 1);
+	ld->domain = domain ? bctbx_strdup(domain) : NULL;
+	ld->logmask = logmask;
+#ifdef THREAD_LOG_LEVEL_ENABLED
+	ld->thread_level_set = FALSE;
+	pthread_key_create(&ld->thread_level_key, thread_level_key_destroy);
+#endif
+	return ld;
+}
+
+void bctbx_log_domain_destroy(BctoolboxLogDomain *obj){
+#if THREAD_LOG_LEVEL_ENABLED
+	pthread_key_delete(obj->thread_level_key);
+#endif	
 	if (obj->domain) bctbx_free(obj->domain);
 	bctbx_free(obj);
 }
 
+#if THREAD_LOG_LEVEL_ENABLED
+unsigned int bctbx_log_domain_get_thread_log_level_mask(BctoolboxLogDomain *ld){
+	unsigned int *specific = (unsigned int*)pthread_getspecific(ld->thread_level_key);
+	if (!specific) return 0;
+	return *specific;
+}
+#endif
+
 typedef struct _bctbx_logger_t {
+	BctoolboxLogDomain *default_log_domain;
 	bctbx_list_t *logv_outs;
-	unsigned int log_mask; /*the default log mask, if no per-domain settings are found*/
 	unsigned long log_thread_id;
 	bctbx_list_t *log_stored_messages_list;
 	bctbx_list_t *log_domains;
 	bctbx_mutex_t log_stored_messages_mutex;
 	bctbx_mutex_t domains_mutex;
 	bctbx_mutex_t log_mutex;
+	bctbx_log_handler_t * default_handler;
 } bctbx_logger_t;
 
 struct _bctbx_log_handler_t {
 	BctbxLogHandlerFunc func;
 	BctbxLogHandlerDestroyFunc destroy;
+	char *domain; /*domain this log handler is limited to. NULL for all*/
 	void* user_info;
 };
 
@@ -71,97 +116,155 @@ typedef struct _bctbx_file_log_handler_t {
 	uint64_t max_size;
 	uint64_t size;
 	FILE* file;
+	bool_t reopen_requested;
 } bctbx_file_log_handler_t;
 
-static bctbx_logger_t __bctbx_logger = { NULL, BCTBX_LOG_WARNING|BCTBX_LOG_ERROR|BCTBX_LOG_FATAL, 0};
 
-static unsigned int bctbx_init_logger_refcount = 0;
+void bctbx_logv_out_cb(void* user_info, const char *domain, BctbxLogLevel lev, const char *fmt, va_list args);
+
+static void wrapper(void* info,const char *domain, BctbxLogLevel lev, const char *fmt, va_list args) {
+	BctbxLogFunc func = (BctbxLogFunc)info;
+	if (func) func(domain, lev, fmt,  args);
+}
+
+static bctbx_logger_t main_logger = {0};
+static bctbx_log_handler_t static_handler = {0};
+
+static bctbx_logger_t * bctbx_get_logger(void){
+	if (main_logger.default_log_domain == NULL){
+		main_logger.default_log_domain = bctbx_log_domain_new(NULL, BCTBX_LOG_WARNING | BCTBX_LOG_ERROR | BCTBX_LOG_FATAL);
+		bctbx_mutex_init(&main_logger.domains_mutex, NULL);
+		bctbx_mutex_init(&main_logger.log_mutex, NULL);
+		main_logger.default_handler = &static_handler;
+		static_handler.func=wrapper;
+		static_handler.destroy=(BctbxLogHandlerDestroyFunc)bctbx_logv_out_destroy;
+		static_handler.user_info=(void*)bctbx_logv_out;
+		bctbx_add_log_handler(&static_handler);
+	}
+	return &main_logger;
+}
 
 void bctbx_init_logger(bool_t create){
-	if (bctbx_init_logger_refcount++ > 0) return; /*already initialized*/
-	
-	bctbx_mutex_init(&__bctbx_logger.domains_mutex, NULL);
-	bctbx_mutex_init(&__bctbx_logger.log_mutex, NULL);
-	if(create) {
-		bctbx_log_handler_t* handler = bctbx_create_log_handler(bctbx_logv_out, bctbx_logv_out_destroy, NULL);
-		bctbx_add_log_handler(handler);
-	}
+	bctbx_get_logger();
 }
 
 void bctbx_log_handlers_free(void) {
-	bctbx_list_t *loggers = bctbx_list_first_elem(__bctbx_logger.logv_outs);
-	while (loggers) {
-		bctbx_log_handler_t* handler = (bctbx_log_handler_t*)loggers->data;
+	bctbx_list_t *handlers = bctbx_list_first_elem(bctbx_get_logger()->logv_outs);
+	while (handlers) {
+		bctbx_log_handler_t* handler = (bctbx_log_handler_t*)handlers->data;
 		handler->destroy(handler);
-		loggers = loggers->next;
+		handlers = handlers->next;
 	}
 }
 
 void bctbx_uninit_logger(void){
-	if (--bctbx_init_logger_refcount <= 0) {
-		bctbx_logv_flush();
-		bctbx_mutex_destroy(&__bctbx_logger.domains_mutex);
-		bctbx_mutex_destroy(&__bctbx_logger.log_mutex);
-		bctbx_log_handlers_free();
-		__bctbx_logger.logv_outs = bctbx_list_free(__bctbx_logger.logv_outs);
-		__bctbx_logger.log_domains = bctbx_list_free_with_data(__bctbx_logger.log_domains, (void (*)(void*))bctbx_log_domain_destroy);
-	}
+	/* Calling bctbx_uninit_logger() from a component is dangerous as it will reset the default logger.
+	 * it is safer that this function does nothing.*/
+#if 0
+	bctbx_logger_t * logger = bctbx_get_logger();
+	bctbx_logv_flush();
+	bctbx_mutex_destroy(&logger->domains_mutex);
+	bctbx_mutex_destroy(&logger->log_mutex);
+	bctbx_log_handlers_free();
+	logger->logv_outs = bctbx_list_free(logger->logv_outs);
+	logger->log_domains = bctbx_list_free_with_data(logger->log_domains, (void (*)(void*))bctbx_log_domain_destroy);
+	bctbx_log_domain_destroy(logger->default_log_domain);
+	logger->default_log_domain = NULL;
+#endif
 }
 
 bctbx_log_handler_t* bctbx_create_log_handler(BctbxLogHandlerFunc func, BctbxLogHandlerDestroyFunc destroy, void* user_info) {
-	bctbx_log_handler_t* handler = (bctbx_log_handler_t*)bctbx_malloc(sizeof(bctbx_log_handler_t));
+	bctbx_log_handler_t* handler = (bctbx_log_handler_t*)bctbx_malloc0(sizeof(bctbx_log_handler_t));
 	handler->func = func;
 	handler->destroy = destroy;
 	handler->user_info = user_info;
 	return handler;
 }
 
-bctbx_log_handler_t* bctbx_create_file_log_handler(uint64_t max_size, const char* path, const char* name, FILE* f) {
-	bctbx_log_handler_t* handler = (bctbx_log_handler_t*)bctbx_malloc0(sizeof(bctbx_log_handler_t));
-	bctbx_file_log_handler_t* filehandler = (bctbx_file_log_handler_t*)bctbx_malloc(sizeof(bctbx_file_log_handler_t));
-	char *full_name = bctbx_strdup_printf("%s/%s",
-									path,
-									name);
-	struct stat buf;
-	memset(&buf, 0, sizeof(buf));
-	handler->func=bctbx_logv_file;
-	handler->destroy=bctbx_logv_file_destroy;
-	filehandler->max_size = max_size;
-	// init with actual file size
-	if(stat(full_name, &buf) != 0) {
-		fprintf(stderr,"Error while creating file log handler. \n");
-		return NULL;
+void bctbx_log_handler_set_user_data(bctbx_log_handler_t* log_handler, void* user_data) {
+	log_handler->user_info = user_data;
+}
+void *bctbx_log_handler_get_user_data(const bctbx_log_handler_t* log_handler) {
+	return log_handler->user_info;
+}
+
+void bctbx_log_handler_set_domain(bctbx_log_handler_t * log_handler, const char *domain) {
+	if (log_handler->domain) bctbx_free(log_handler->domain);
+	if (domain) {
+		log_handler->domain = bctbx_strdup(domain);
+	} else {
+		log_handler->domain = NULL ;
 	}
-	bctbx_free(full_name);
+	
+}
+bctbx_log_handler_t* bctbx_create_file_log_handler(uint64_t max_size, const char* path, const char* name) {
+	bctbx_log_handler_t *handler = NULL;
+	bctbx_file_log_handler_t *filehandler = NULL;
+	char *full_name = bctbx_strdup_printf("%s/%s", path, name);
+	struct stat buf = {0};
+
+	FILE *f = fopen(full_name, "a");
+	if (f == NULL) {
+		fprintf(stderr, "error while opening '%s': %s\n", full_name, strerror(errno));
+		goto end;
+	}
+	if (stat(full_name, &buf) != 0) {
+		fprintf(stderr, "error while gathering info about '%s': %s", full_name, strerror(errno));
+		goto end;
+	}
+
+	filehandler = bctbx_new0(bctbx_file_log_handler_t, 1);
+	filehandler->max_size = max_size;
 	filehandler->size = buf.st_size;
 	filehandler->path = bctbx_strdup(path);
 	filehandler->name = bctbx_strdup(name);
 	filehandler->file = f;
-	handler->user_info=(void*) filehandler;
+
+	handler = bctbx_new0(bctbx_log_handler_t, 1);
+	handler->func = bctbx_logv_file;
+	handler->destroy = bctbx_logv_file_destroy;
+	handler->user_info = filehandler;
+
+end:
+	bctbx_free(full_name);
 	return handler;
 }
 
+void bctbx_file_log_handler_reopen(bctbx_log_handler_t *file_log_handler) {
+	bctbx_file_log_handler_t *filehandler = (bctbx_file_log_handler_t *)file_log_handler->user_info;
+	bctbx_logger_t *logger = bctbx_get_logger();
+	bctbx_mutex_lock(&logger->log_mutex);
+	filehandler->reopen_requested = TRUE;
+	bctbx_mutex_unlock(&logger->log_mutex);
+}
+
 /**
 *@param func: your logging function, compatible with the BctoolboxLogFunc prototype.
 *
 **/
 void bctbx_add_log_handler(bctbx_log_handler_t* handler){
-	if (handler && !bctbx_list_find(__bctbx_logger.logv_outs, handler))
-		__bctbx_logger.logv_outs = bctbx_list_append(__bctbx_logger.logv_outs, (void*)handler);
+	bctbx_logger_t *logger = bctbx_get_logger();
+	if (handler && !bctbx_list_find(logger->logv_outs, handler))
+		logger->logv_outs = bctbx_list_append(logger->logv_outs, (void*)handler);
 	/*else, already in*/
 }
 
-static void wrapper(void* info,const char *domain, BctbxLogLevel lev, const char *fmt, va_list args) {
-	BctbxLogFunc func = (BctbxLogFunc)info;
-	func(domain, lev, fmt,  args);
+void bctbx_remove_log_handler(bctbx_log_handler_t* handler){
+	bctbx_logger_t *logger = bctbx_get_logger();
+	logger->logv_outs = bctbx_list_remove(logger->logv_outs,  handler);
+	handler->destroy(handler);
+	return;
 }
 
+
 void bctbx_set_log_handler(BctbxLogFunc func){
-	static bctbx_log_handler_t handler;
-	handler.func=wrapper;
-	handler.destroy=(BctbxLogHandlerDestroyFunc)bctbx_logv_out_destroy;
-	handler.user_info=(void*)func;
-	bctbx_add_log_handler(&handler);
+	bctbx_set_log_handler_for_domain(func,NULL);
+}
+
+void bctbx_set_log_handler_for_domain(BctbxLogFunc func, const char* domain){
+	bctbx_log_handler_t *h = bctbx_get_logger()->default_handler;
+	h->user_info=(void*)func;
+	bctbx_log_handler_set_domain(h, domain);
 }
 
 void bctbx_set_log_file(FILE* f){
@@ -176,14 +279,15 @@ void bctbx_set_log_file(FILE* f){
 }
 
 bctbx_list_t* bctbx_get_log_handlers(void){
-	return __bctbx_logger.logv_outs;
+	return bctbx_get_logger()->logv_outs;
 }
 
 static BctoolboxLogDomain * get_log_domain(const char *domain){
 	bctbx_list_t *it;
+	bctbx_logger_t *logger = bctbx_get_logger();
 
-	if (domain == NULL) return NULL;
-	for (it = __bctbx_logger.log_domains; it != NULL; it = bctbx_list_next(it)) {
+	if (domain == NULL) return logger->default_log_domain;
+	for (it = logger->log_domains; it != NULL; it = bctbx_list_next(it)) {
 		BctoolboxLogDomain *ld = (BctoolboxLogDomain*)bctbx_list_get_data(it);
 		if (ld->domain && strcmp(ld->domain, domain) == 0 ){
 			return ld;
@@ -194,20 +298,18 @@ static BctoolboxLogDomain * get_log_domain(const char *domain){
 
 static BctoolboxLogDomain *get_log_domain_rw(const char *domain){
 	BctoolboxLogDomain *ret;
-
-	if (domain == NULL) return NULL;
+	bctbx_logger_t *logger = bctbx_get_logger();
+	
 	ret = get_log_domain(domain);
 	if (ret) return ret;
 	/*it does not exist, hence create it by taking the mutex*/
-	bctbx_mutex_lock(&__bctbx_logger.domains_mutex);
+	bctbx_mutex_lock(&logger->domains_mutex);
 	ret = get_log_domain(domain);
 	if (!ret){
-		ret = bctbx_new0(BctoolboxLogDomain,1);
-		ret->domain = bctbx_strdup(domain);
-		ret->logmask = __bctbx_logger.log_mask;
-		__bctbx_logger.log_domains = bctbx_list_prepend(__bctbx_logger.log_domains, ret);
+		ret = bctbx_log_domain_new(domain, logger->default_log_domain->logmask);
+		logger->log_domains = bctbx_list_prepend(logger->log_domains, ret);
 	}
-	bctbx_mutex_unlock(&__bctbx_logger.domains_mutex);
+	bctbx_mutex_unlock(&logger->domains_mutex);
 	return ret;
 }
 
@@ -216,13 +318,11 @@ static BctoolboxLogDomain *get_log_domain_rw(const char *domain){
 * BCTBX_FATAL .
 **/
 void bctbx_set_log_level_mask(const char *domain, int levelmask){
-	if (domain == NULL) __bctbx_logger.log_mask=levelmask;
-	else get_log_domain_rw(domain)->logmask = levelmask;
+	get_log_domain_rw(domain)->logmask = levelmask;
 }
 
-
-void bctbx_set_log_level(const char *domain, BctbxLogLevel level){
-	int levelmask = BCTBX_LOG_FATAL;
+static unsigned int level_to_mask(BctbxLogLevel level){
+	unsigned int levelmask = BCTBX_LOG_FATAL;
 	if (level<=BCTBX_LOG_ERROR){
 		levelmask |= BCTBX_LOG_ERROR;
 	}
@@ -238,23 +338,40 @@ void bctbx_set_log_level(const char *domain, BctbxLogLevel level){
 	if (level<=BCTBX_LOG_DEBUG){
 		levelmask |= BCTBX_LOG_DEBUG;
 	}
-	bctbx_set_log_level_mask(domain, levelmask);
+	return levelmask;
+}
+
+void bctbx_set_log_level(const char *domain, BctbxLogLevel level){
+	bctbx_set_log_level_mask(domain, level_to_mask(level));
 }
 
 unsigned int bctbx_get_log_level_mask(const char *domain) {
-	BctoolboxLogDomain *ld;
-	if (domain == NULL || (ld = get_log_domain(domain)) == NULL) return __bctbx_logger.log_mask;
-	else return ld->logmask;
+	BctoolboxLogDomain *ld = get_log_domain(domain);
+	if (!ld) ld = bctbx_get_logger()->default_log_domain;
+	return ld->logmask;
+}
+
+int bctbx_log_level_enabled(const char *domain, BctbxLogLevel level){
+	unsigned int logmask = 0;
+	BctoolboxLogDomain *ld = get_log_domain(domain);
+	if (!ld) ld = bctbx_get_logger()->default_log_domain;
+	
+#ifdef THREAD_LOG_LEVEL_ENABLED
+	if (ld->thread_level_set) logmask = bctbx_log_domain_get_thread_log_level_mask(ld);
+#endif
+	if (logmask == 0) logmask = ld->logmask; /* if there is no thread specific log level, revert to global log level.*/
+	return (logmask & (unsigned int)level) != 0;
 }
 
 void bctbx_set_log_thread_id(unsigned long thread_id) {
+	bctbx_logger_t *logger = bctbx_get_logger();
 	if (thread_id == 0) {
 		bctbx_logv_flush();
-		bctbx_mutex_destroy(&__bctbx_logger.log_stored_messages_mutex);
+		bctbx_mutex_destroy(&logger->log_stored_messages_mutex);
 	} else {
-		bctbx_mutex_init(&__bctbx_logger.log_stored_messages_mutex, NULL);
+		bctbx_mutex_init(&logger->log_stored_messages_mutex, NULL);
 	}
-	__bctbx_logger.log_thread_id = thread_id;
+	logger->log_thread_id = thread_id;
 }
 
 char * bctbx_strdup_vprintf(const char *fmt, va_list ap)
@@ -316,8 +433,7 @@ char * bctbx_strcat_vprintf(char* dst, const char *fmt, va_list ap){
 	retlen = strlen(ret);
 
 	if ((dst = bctbx_realloc(dst, dstlen+retlen+1)) != NULL){
-		strncat(dst,ret,retlen);
-		dst[dstlen+retlen] = '\0';
+		strcat(dst,ret);
 		bctbx_free(ret);
 		return dst;
 	} else {
@@ -352,35 +468,38 @@ void _bctbx_logv_flush(int dummy, ...) {
 	bctbx_list_t *msglist;
 	va_list empty_va_list;
 	va_start(empty_va_list, dummy);
-	bctbx_mutex_lock(&__bctbx_logger.log_stored_messages_mutex);
-	msglist = __bctbx_logger.log_stored_messages_list;
-	__bctbx_logger.log_stored_messages_list = NULL;
-	bctbx_mutex_unlock(&__bctbx_logger.log_stored_messages_mutex);
+	
+	bctbx_logger_t *logger = bctbx_get_logger();
+	
+	bctbx_mutex_lock(&logger->log_stored_messages_mutex);
+	msglist = logger->log_stored_messages_list;
+	logger->log_stored_messages_list = NULL;
+	bctbx_mutex_unlock(&logger->log_stored_messages_mutex);
 	for (elem = msglist; elem != NULL; elem = bctbx_list_next(elem)) {
 		bctbx_stored_log_t *l = (bctbx_stored_log_t *)bctbx_list_get_data(elem);
-		bctbx_list_t *loggers = bctbx_list_first_elem(__bctbx_logger.logv_outs);
+		bctbx_list_t *handlers = bctbx_list_first_elem(logger->logv_outs);
 #ifdef _WIN32
-		while (loggers) {
-			bctbx_log_handler_t* handler = (bctbx_log_handler_t*)loggers->data;
+		while (handlers) {
+			bctbx_log_handler_t* handler = (bctbx_log_handler_t*)handlers->data;
 			if(handler) {
 				va_list cap;
 				va_copy(cap, empty_va_list);
 				handler->func(handler->user_info, l->domain, l->level, l->msg, cap);
 				va_end(cap);
 			}
-			loggers = loggers->next;
+			handlers = handlers->next;
 		}
 #else
 		
-		while (loggers) {
-			bctbx_log_handler_t* handler = (bctbx_log_handler_t*)loggers->data;
+		while (handlers) {
+			bctbx_log_handler_t* handler = (bctbx_log_handler_t*)handlers->data;
 			if(handler) {
 				va_list cap;
 				va_copy(cap, empty_va_list);
 				handler->func(handler->user_info, l->domain, l->level, l->msg, cap);
 				va_end(cap);
 			}
-			loggers = loggers->next;
+			handlers = handlers->next;
 		}
 		
 #endif
@@ -397,53 +516,62 @@ void bctbx_logv_flush(void) {
 }
 
 void bctbx_logv(const char *domain, BctbxLogLevel level, const char *fmt, va_list args) {
-	if ((__bctbx_logger.logv_outs != NULL) && bctbx_log_level_enabled(domain, level)) {
-		if (__bctbx_logger.log_thread_id == 0) {
-			bctbx_list_t *loggers = bctbx_list_first_elem(__bctbx_logger.logv_outs);
-			while (loggers) {
-				bctbx_log_handler_t* handler = (bctbx_log_handler_t*)loggers->data;
-				if(handler) {
+	bctbx_logger_t *logger = bctbx_get_logger();
+	
+	if ((logger->logv_outs != NULL) && bctbx_log_level_enabled(domain, level)) {
+		if (logger->log_thread_id == 0) {
+			bctbx_list_t *handlers = bctbx_list_first_elem(logger->logv_outs);
+			while (handlers) {
+				bctbx_log_handler_t* handler = (bctbx_log_handler_t*)handlers->data;
+				if(handler && (!handler->domain || !domain || strcmp(handler->domain,domain)==0)) {
 					va_list tmp;
 					va_copy(tmp, args);
 					handler->func(handler->user_info, domain, level, fmt, tmp);
 					va_end(tmp);
 				}
-				loggers = loggers->next;
+				handlers = handlers->next;
 			}
-		} else if (__bctbx_logger.log_thread_id == bctbx_thread_self()) {
-			bctbx_list_t *loggers;
+		} else if (logger->log_thread_id == bctbx_thread_self()) {
+			bctbx_list_t *handlers;
 			bctbx_logv_flush();
-			loggers = bctbx_list_first_elem(__bctbx_logger.logv_outs);
-			while (loggers) {
-				bctbx_log_handler_t* handler = (bctbx_log_handler_t*)loggers->data;
-				if(handler) {
+			handlers = bctbx_list_first_elem(logger->logv_outs);
+			while (handlers) {
+				bctbx_log_handler_t* handler = (bctbx_log_handler_t*)handlers->data;
+				if(handler && (!handler->domain || !domain || strcmp(handler->domain,domain)==0)) {
 					va_list tmp;
 					va_copy(tmp, args);
 					handler->func(handler->user_info, domain, level, fmt, tmp);
 					va_end(tmp);
 				}
-				loggers = loggers->next;
+				handlers = handlers->next;
 			}
 		} else {
 			bctbx_stored_log_t *l = bctbx_new(bctbx_stored_log_t, 1);
 			l->domain = domain ? bctbx_strdup(domain) : NULL;
 			l->level = level;
 			l->msg = bctbx_strdup_vprintf(fmt, args);
-			bctbx_mutex_lock(&__bctbx_logger.log_stored_messages_mutex);
-			__bctbx_logger.log_stored_messages_list = bctbx_list_append(__bctbx_logger.log_stored_messages_list, l);
-			bctbx_mutex_unlock(&__bctbx_logger.log_stored_messages_mutex);
+			bctbx_mutex_lock(&logger->log_stored_messages_mutex);
+			logger->log_stored_messages_list = bctbx_list_append(logger->log_stored_messages_list, l);
+			bctbx_mutex_unlock(&logger->log_stored_messages_mutex);
 		}
 	}
 #if !defined(_WIN32_WCE)
 	if (level == BCTBX_LOG_FATAL) {
 		bctbx_logv_flush();
+#ifdef __ANDROID__
+        //Act as a flush + abort
+		__android_log_assert(NULL, NULL, "%s", "Aborting");
+#else
 		abort();
+#endif
 	}
 #endif
 }
-
+void bctbx_logv_out( const char *domain, BctbxLogLevel lev, const char *fmt, va_list args){
+	bctbx_logv_out_cb(NULL, domain, lev, fmt, args);
+}
 /*This function does the default formatting and output to file*/
-void bctbx_logv_out(void* user_info, const char *domain, BctbxLogLevel lev, const char *fmt, va_list args){
+void bctbx_logv_out_cb(void* user_info, const char *domain, BctbxLogLevel lev, const char *fmt, va_list args){
 	const char *lname="undef";
 	char *msg;
 	struct timeval tp;
@@ -548,14 +676,16 @@ static void _rotate_log_collection_files(bctbx_file_log_handler_t *filehandler)
 	while(access(log_filename, F_OK) != -1) {
 		// file exists
 		n++;
+		bctbx_free(log_filename);
 		log_filename = bctbx_strdup_printf("%s/%s_%d%s",
 		filehandler->path,
 		file_no_extension,
 		n,
 		extension2);
 	}
-
+	
 	while(n > 1) {
+		bctbx_free(log_filename);
 		log_filename = bctbx_strdup_printf("%s/%s_%d%s",
 		filehandler->path,
 		file_no_extension,
@@ -569,8 +699,9 @@ static void _rotate_log_collection_files(bctbx_file_log_handler_t *filehandler)
 
 		n--;
 		rename(log_filename, log_filename2);
+		bctbx_free(log_filename2);
 	}
-
+	bctbx_free(log_filename);
 	log_filename = bctbx_strdup_printf("%s/%s",
 	filehandler->path,
 	filehandler->name);
@@ -602,7 +733,7 @@ static void _close_log_collection_file(bctbx_file_log_handler_t *filehandler) {
 
 void bctbx_logv_file(void* user_info, const char *domain, BctbxLogLevel lev, const char *fmt, va_list args){
 	const char *lname="undef";
-	char *msg;
+	char *msg = NULL;
 	struct timeval tp;
 	struct tm *lt;
 #ifndef _WIN32
@@ -611,9 +742,10 @@ void bctbx_logv_file(void* user_info, const char *domain, BctbxLogLevel lev, con
 	time_t tt;
 	int ret = -1;
 	bctbx_file_log_handler_t *filehandler = (bctbx_file_log_handler_t *) user_info;
-	FILE *f;
-	bctbx_mutex_lock(&__bctbx_logger.log_mutex);
-	f = filehandler->file;
+	bctbx_logger_t *logger = bctbx_get_logger();
+	
+	bctbx_mutex_lock(&logger->log_mutex);
+	FILE *f = filehandler ? filehandler->file : stdout;
 	bctbx_gettimeofday(&tp,NULL);
 	tt = (time_t)tp.tv_sec;
 
@@ -623,9 +755,8 @@ void bctbx_logv_file(void* user_info, const char *domain, BctbxLogLevel lev, con
 	lt = localtime_r(&tt,&tmbuf);
 #endif
 
-	if(!f) {
-		return;
-	}
+	if(!f) goto end;
+
 	switch(lev){
 		case BCTBX_LOG_DEBUG:
 			lname = "debug";
@@ -666,16 +797,26 @@ void bctbx_logv_file(void* user_info, const char *domain, BctbxLogLevel lev, con
 			,1900+lt->tm_year,1+lt->tm_mon,lt->tm_mday,lt->tm_hour,lt->tm_min,lt->tm_sec
 		,(int)(tp.tv_usec/1000), (domain?domain:"bctoolbox"), lname, msg);
 	fflush(f);
-	if (filehandler->max_size > 0 && ret > 0) {
-		filehandler->size += ret;
-		if (filehandler->size > filehandler->max_size) {
+
+	/* reopen the log file when either the size limit has been exceeded, or reopen has been required
+	   by the user. Reopening a log file that has reached the size limit automatically trigger log rotation
+	   while opening. */
+	if (filehandler) {
+		bool_t reopen_requested = filehandler->reopen_requested;
+		if (filehandler->max_size > 0 && ret > 0) {
+			filehandler->size += ret;
+			reopen_requested = reopen_requested || filehandler->size > filehandler->max_size;
+		}
+		if (reopen_requested) {
 			_close_log_collection_file(filehandler);
 			_open_log_collection_file(filehandler);
+			filehandler->reopen_requested = FALSE;
 		}
 	}
-	bctbx_mutex_unlock(&__bctbx_logger.log_mutex);
 
-	bctbx_free(msg);
+end:
+	bctbx_mutex_unlock(&logger->log_mutex);
+	if (msg) bctbx_free(msg);
 }
 
 void bctbx_logv_file_destroy(bctbx_log_handler_t* handler) {
@@ -686,6 +827,27 @@ void bctbx_logv_file_destroy(bctbx_log_handler_t* handler) {
 	bctbx_logv_out_destroy(handler);
 }
 
+void bctbx_set_thread_log_level(const char *domain, BctbxLogLevel level){
+#ifdef THREAD_LOG_LEVEL_ENABLED
+	BctoolboxLogDomain * ld = get_log_domain(domain);
+	unsigned int *specific = (unsigned int*)pthread_getspecific(ld->thread_level_key);
+	if (!specific) specific = bctbx_new0(unsigned int, 1);
+	*specific = level_to_mask(level);
+	pthread_setspecific(ld->thread_level_key, specific);
+	ld->thread_level_set = TRUE;
+#endif
+}
+
+
+void bctbx_clear_thread_log_level(const char *domain){
+#ifdef THREAD_LOG_LEVEL_ENABLED
+	BctoolboxLogDomain * ld = get_log_domain(domain);
+	unsigned int *specific = (unsigned int*)pthread_getspecific(ld->thread_level_key);
+	if (specific) *specific = 0;
+#endif
+}
+
+
 #ifdef __QNX__
 #include <slog2.h>
 
