@@ -1,22 +1,22 @@
 
 /*
- bctoolbox
- Copyright (C) 2016  Belledonne Communications SARL
-
-  This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Lesser General Public
-  License as published by the Free Software Foundation; either
-  version 2.1 of the License, or (at your option) any later version.
-
-  This library is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Lesser General Public License for more details.
-
-  You should have received a copy of the GNU Lesser General Public
-  License along with this library; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*/
+ * Copyright (c) 2016-2020 Belledonne Communications SARL.
+ *
+ * This file is part of bctoolbox.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -108,33 +108,15 @@ char * bctbx_strdup(const char *tmp){
 }
 
 char * bctbx_dirname(const char *path) {
-	char *ptr;
-	char *dname = bctbx_strdup(path);
-	bool_t found = FALSE;
-
-	ptr = strrchr(path, '/');
-	if (ptr != NULL) {
-		dname[ptr - path] = '\0';
-		found = TRUE;
-	} else {
-		ptr = strrchr(path, '\\');
-		if (ptr != NULL) {
-			dname[ptr - path] = '\0';
-			found = TRUE;
-		}
-	}
-
-	if (found == FALSE) {
-		bctbx_free(dname);
-		return NULL;
-	}
-	return dname;
+	char *ptr = strrchr(path, '/');
+	if (ptr == NULL) ptr = strrchr(path, '\\');
+	return ptr ? bctbx_strndup(path, ptr-path) : bctbx_strdup(".");
 }
 
 char * bctbx_basename(const char *path) {
 	char *ptr = strrchr(path, '/');
 	if (ptr == NULL) ptr = strrchr(path, '\\');
-	if (ptr == NULL) return NULL;
+	if (ptr == NULL) return bctbx_strdup(path);
 	return bctbx_strdup(ptr + 1);
 }
 
@@ -158,6 +140,15 @@ int bctbx_file_exist(const char *pathname) {
 	return access(pathname,F_OK);
 }
 
+bool_t bctbx_directory_exists(const char *pathname) {
+	struct stat sb;
+#ifdef WIN32
+	return stat(pathname, &sb) == 0 && (_S_IFDIR & sb.st_mode);
+#else
+	return stat(pathname, &sb) == 0 && S_ISDIR(sb.st_mode);
+#endif
+}
+
 #if	!defined(_WIN32) && !defined(_WIN32_WCE)
 	/* Use UNIX inet_aton method */
 #else
@@ -714,7 +705,7 @@ void bctbx_shm_close(void *mem){
 
 void _bctbx_get_cur_time(bctoolboxTimeSpec *ret, bool_t realtime){
 #if defined(_WIN32_WCE) || defined(WIN32)
-#ifdef BCTBX_WINDOWS_DESKTOP
+#if defined(BCTBX_WINDOWS_DESKTOP) && !defined(ENABLE_MICROSOFT_STORE_APP)
 	DWORD timemillis;
 #	if defined(_WIN32_WCE)
 	timemillis=GetTickCount();
@@ -964,9 +955,7 @@ static int bctbx_wincrypto_random(unsigned int *rand_number){
 #endif
 
 unsigned int bctbx_random(void){
-#ifdef HAVE_ARC4RANDOM
-	return arc4random();
-#elif  defined(__linux) || defined(__APPLE__)
+#if defined(__linux) || defined(__APPLE__)
 	static int fd=-1;
 	if (fd==-1) fd=open("/dev/urandom",O_RDONLY);
 	if (fd!=-1){
@@ -975,6 +964,11 @@ unsigned int bctbx_random(void){
 			bctbx_error("Reading /dev/urandom failed.");
 		}else return tmp;
 	}else bctbx_error("Could not open /dev/urandom");
+#	ifdef HAVE_ARC4RANDOM
+	return arc4random(); // fallback to arc4random().
+#	endif
+#elif defined(HAVE_ARC4RANDOM)
+	return arc4random();
 #elif defined(_WIN32)
 	static int initd=0;
 	unsigned int ret;
@@ -1004,6 +998,7 @@ unsigned int bctbx_random(void){
 	return (unsigned int) random();
 #endif
 }
+
 bool_t bctbx_is_multicast_addr(const struct sockaddr *addr) {
 	
 	switch (addr->sa_family) {
@@ -1216,6 +1211,50 @@ static void _bctbx_addrinfo_to_ip_address_error(int err, char *ip, size_t ip_siz
 	strncpy(ip, "<bug!!>", ip_size);
 }
 
+struct addrinfo* bctbx_addrinfo_sort(struct addrinfo *ais) {
+	bctbx_list_t* v6 = NULL;
+	bctbx_list_t* v4_mapped = NULL;
+	bctbx_list_t* v4 = NULL;
+	bctbx_list_t *it = NULL;
+	struct addrinfo* res0 = NULL;
+	struct addrinfo* res = NULL;
+	struct addrinfo* ai = NULL;
+	
+	//sort by type
+	for (ai = ais; ai != NULL;  ) {
+		struct addrinfo* next = ai->ai_next;
+		struct sockaddr_in6 *sock_in6 = (struct sockaddr_in6 *)ai->ai_addr;
+		if (ai->ai_family == AF_INET6) {
+			if (IN6_IS_ADDR_V4MAPPED(&sock_in6->sin6_addr)) {
+				v4_mapped = bctbx_list_prepend(v4_mapped, ai);
+			} else {
+				v6 = bctbx_list_prepend(v6, ai);
+			}
+		} else {
+				v4 = bctbx_list_prepend(v4, ai);
+		}
+		
+		ai->ai_next = NULL ;
+		ai = next;
+	}
+	v6 = bctbx_list_concat(v6, v4_mapped);
+	v6 = bctbx_list_concat(v6, v4);
+	
+	for (it = v6; it != NULL; it = it->next) {
+		if (res0 == NULL) {
+			res0 = res = (struct addrinfo*)it->data;
+		} else {
+			res->ai_next = (struct addrinfo*)it->data;
+			res = res->ai_next;
+ 		}
+	}
+	if (res)
+		res->ai_next = NULL;
+	
+	bctbx_list_free(v6);
+	
+	return res0;
+}
 int bctbx_addrinfo_to_ip_address(const struct addrinfo *ai, char *ip, size_t ip_size, int *port){
 	char serv[16];
 	int err=bctbx_getnameinfo(ai->ai_addr,(socklen_t)ai->ai_addrlen,ip,(socklen_t)ip_size,serv,(socklen_t)sizeof(serv),NI_NUMERICHOST|NI_NUMERICSERV);
@@ -1279,11 +1318,62 @@ static struct addrinfo * _bctbx_name_to_addrinfo(int family, int socktype, const
 			bctbx_error("%s(%s): getaddrinfo failed: %s",__FUNCTION__, ipaddress, gai_strerror(err));
 		return NULL;
 	}
+	//sort result
+	if (res)
+		res = bctbx_addrinfo_sort(res);
+	
 	return res;
 }
 
 struct addrinfo * bctbx_name_to_addrinfo(int family, int socktype, const char *name, int port){
-	return _bctbx_name_to_addrinfo(family, socktype, name, port, FALSE);
+	struct addrinfo * res = NULL;
+#if defined(__ANDROID__)
+	// This is to workaround possible ANR on Android
+	/*"main" prio=5 tid=1 Native
+  	| group="main" sCount=1 dsCount=0 obj=0x75d55258 self=0x231c03fa00
+  	| sysTid=27931 nice=-10 cgrp=default sched=0/0 handle=0x2320165a98
+  	| state=S schedstat=( 0 0 0 ) utm=303 stm=69 core=1 HZ=100
+  	| stack=0x48ae2c5000-0x48ae2c7000 stackSize=8MB
+  	| held mutexes=
+  	kernel: __switch_to+0x70/0x7c
+  	kernel: unix_stream_recvmsg+0x254/0x6f8
+  	kernel: sock_aio_read.part.9+0xe4/0x110
+  	kernel: sock_aio_read+0x20/0x30
+  	kernel: do_sync_read+0x70/0xa8
+  	kernel: vfs_read+0xb0/0x140
+  	kernel: SyS_read+0x54/0xa4
+  	kernel: el0_svc_naked+0x24/0x28
+  	native: #00 pc 000000000006b6e4  /system/lib64/libc.so (read+4)
+  	native: #01 pc 00000000000731f8  /system/lib64/libc.so (__sread+44)
+  	native: #02 pc 0000000000076fac  /system/lib64/libc.so (__srefill+260)
+  	native: #03 pc 0000000000076e00  /system/lib64/libc.so (fread+272)
+  	native: #04 pc 000000000002fcf0  /system/lib64/libc.so (android_getaddrinfofornetcontext+2356)
+  	native: #05 pc 000000000002f33c  /system/lib64/libc.so (getaddrinfo+56)
+  	native: #06 pc 000000000001e754  /data/app/com.meetme-1/lib/arm64/libbctoolbox.so (bctbx_getaddrinfo+164)
+  	native: #07 pc 000000000001ecb4  /data/app/com.meetme-1/lib/arm64/libbctoolbox.so (???)
+  	native: #08 pc 0000000000018edc  /data/app/com.meetme-1/lib/arm64/libortp.so (???)
+  	native: #09 pc 000000000009dd10  /data/app/com.meetme-1/lib/arm64/libmediastreamer_voip.so (audio_stream_start_from_io+132)
+  	native: #10 pc 00000000007948d0  /data/app/com.meetme-1/lib/arm64/liblinphone.so (_ZN15LinphonePrivate19MediaSessionPrivate16startAudioStreamENS_11CallSession5StateEb+3508)
+  	native: #11 pc 00000000007956e8  /data/app/com.meetme-1/lib/arm64/liblinphone.so (_ZN15LinphonePrivate19MediaSessionPrivate12startStreamsENS_11CallSession5StateE+944)
+  	native: #12 pc 00000000007839cc  /data/app/com.meetme-1/lib/arm64/liblinphone.so (_ZN15LinphonePrivate19MediaSessionPrivate13updateStreamsEP19SalMediaDescriptionNS_11CallSession5StateE+692)
+  	native: #13 pc 000000000078513c  /data/app/com.meetme-1/lib/arm64/liblinphone.so (_ZN15LinphonePrivate19MediaSessionPrivate13remoteRingingEv+932)
+  	native: #14 pc 0000000000852f9c  /data/app/com.meetme-1/lib/arm64/liblinphone.so (???)
+  	native: #15 pc 00000000007d79c4  /data/app/com.meetme-1/lib/arm64/liblinphone.so (_ZN15LinphonePrivate9SalCallOp17processResponseCbEPvPK24belle_sip_response_event+788)
+  	native: #16 pc 00000000007e4bb4  /data/app/com.meetme-1/lib/arm64/liblinphone.so (_ZN15LinphonePrivate3Sal22processResponseEventCbEPvPK24belle_sip_response_event+1092)
+  	native: #17 pc 00000000008fdd2c  /data/app/com.meetme-1/lib/arm64/liblinphone.so (belle_sip_client_transaction_notify_response+520)
+  	native: #18 pc 00000000008f7494  /data/app/com.meetme-1/lib/arm64/liblinphone.so (belle_sip_provider_dispatch_message+1376)
+	native: #19 pc 00000000008e0760  /data/app/com.meetme-1/lib/arm64/liblinphone.so (???)
+  	native: #20 pc 00000000008dee1c  /data/app/com.meetme-1/lib/arm64/liblinphone.so (belle_sip_channel_process_data+344)
+  	native: #21 pc 00000000008d5b10  /data/app/com.meetme-1/lib/arm64/liblinphone.so (belle_sip_main_loop_run+772)
+  	native: #22 pc 00000000008d5d54  /data/app/com.meetme-1/lib/arm64/liblinphone.so (belle_sip_main_loop_sleep+72)
+	native: #23 pc 000000000086b9f8  /data/app/com.meetme-1/lib/arm64/liblinphone.so (linphone_core_iterate+540)
+  	native: #24 pc 00000000000743c8  /data/app/com.meetme-1/oat/arm64/base.odex (Java_org_linphone_core_CoreImpl_iterate__J+132)*/
+	res = _bctbx_name_to_addrinfo(family, socktype, name, port, TRUE);
+#endif
+	if (res == NULL) {
+		res = _bctbx_name_to_addrinfo(family, socktype, name, port, FALSE);
+	}
+	return res;
 }
 
 struct addrinfo * bctbx_ip_address_to_addrinfo(int family, int socktype, const char *name, int port){
@@ -1368,6 +1458,18 @@ void bctbx_sockaddr_ipv4_to_ipv6(const struct sockaddr *v4, struct sockaddr *res
 	}
 }
 
+char *bctbx_replace(char *str, char c, char n) {
+	char *s = str;
+
+	while (*s) {
+		if (*s == c) {
+			*s = n;
+		}
+		++s;
+	}
+	return str;
+}
+
 char * bctbx_concat(const char *str, ...) {
 	va_list ap;
 	size_t allocated = 100;
@@ -1676,3 +1778,13 @@ uint64_t bctbx_str_to_uint64(const uint8_t input_string[17]) {
 		| (((uint64_t)bctbx_char_to_byte(input_string[14]))<<4)
 		| (((uint64_t)bctbx_char_to_byte(input_string[15])));
 }
+
+#if defined(__ANDROID__)
+int mblen(const char* s, size_t n) {
+  mbstate_t state = {};
+  return (int)mbrlen(s, n, &state);
+}
+int wctomb(char *s, wchar_t wc) { 
+  return wcrtomb(s,wc,NULL); 
+}
+#endif
